function [m,expplot,EXP]=linearLyapunov2(xx,mode)
% function [m,expplot,EXP]=linearLyapunov2(xx,mode)
% mode=0: use CWM's training result, mode=1: use gradient descent's training result

switch(mode)
    case 0
      load MIMOcwmmodel mux varx beta pcm;
    case 1
      load sseMIMO mux varx beta pcm;
   otherwise
    disp('mode has to be either 0 or 1');
end

% Specify xx, yy and M first
d=length(xx(:,1));
N=length(xx(1,:));
M=length(mux(1,:));
 f=ones(d+1,N);q=2*pi; sqi=(sqrt(q))^d;
 px_cm=zeros(N,M); 
 for i=2:(d+1)
    for n=1:N
        f(i,n)=xx(i-1,n);
    end
end
 
 for n=1:N
    for m=1:M
           Vxi=varx(:,m).^-1; 
           t=xx(:,n)-mux(:,m);
           t=t.^2;
           px_cm(n,m)=exp(-t'*Vxi*0.5)/(sqrt(prod(varx(:,m)))*(sqi));
    end
 end
%  %%%%%%%% Jocobian %%%%%%%%%%%%%%%%
Df=zeros(d,d); 
 Q=eye(d);
 EXP=zeros(1,d);
 K=N;


for n=1:N
 % calculate Df(n)   
for i=1:d
   for j=1:d
       A=0; B=0; Ap=0; Bp=0;
       for m=1:M
           temp=px_cm(n,m)*pcm(m);
           A=A+f(:,n)'*beta(:,m,i)*temp;
           B=B+temp;
%       end
%       for m=1:M
          px_cmp=-px_cm(n,m)*((xx(j,n)-mux(j,m))/varx(j,m));
          Ap=Ap+beta(j+1,m,i)*px_cm(n,m)*pcm(m)+f(:,n)'*beta(:,m,i)*px_cmp*pcm(m);
          Bp=Bp+px_cmp*pcm(m);
      end
          
      Df(i,j)=(Ap*B-Bp*A)/B^2;
  end
end

[Q R]=qr(Df*Q);
 EXP=EXP+log(diag(abs(R)))';
 expplot(n)=max(EXP)/n;
end  % end of for n=1:N

%%% Evaluate finite-time Lyapunov exponent
EXP=EXP/K;
m=max(EXP);  % Calculate the maximal Lyapunov exponent among all d dimensions 

